"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _path = _interopRequireDefault(require("path"));

var _core = require("@babel/core");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _default = (0, _helperPluginUtils.declare)(function (api) {
  api.assertVersion(7);

  function addDisplayName(id, call) {
    var objectAssignExpression = _core.types.memberExpression(_core.types.identifier("Object"), _core.types.identifier("assign"));

    var callExpression = _core.types.callExpression(objectAssignExpression, // We're calling 'Object.assign'.
    [_core.types.objectExpression([]), call, _core.types.objectExpression([_core.types.objectProperty(_core.types.identifier("displayName"), _core.types.stringLiteral(id))])] // Call 'Object.assign'.
    );

    return callExpression;
  }

  var isStyledFn = function isStyledFn(callee) {
    return callee.name === "styled";
  };

  function isStyled(node) {
    if (!node || !_core.types.isCallExpression(node)) return false; // not styled member object

    if (!isStyledFn(node.callee)) {
      return false;
    } // no call arguments


    var args = node.arguments;
    if (args.length < 2) return false;
    return true;
  }

  return {
    name: "transform-react-display-name",
    visitor: {
      ExportDefaultDeclaration: function ExportDefaultDeclaration(nodePath, state) {
        var node = nodePath.node;

        if (isStyled(node.declaration)) {
          var filename = state.filename || "unknown";

          var displayName = _path["default"].basename(filename, _path["default"].extname(filename)); // ./{module name}/index.js


          if (displayName === "index") {
            displayName = _path["default"].basename(_path["default"].dirname(filename));
          }

          nodePath.replaceWith(_core.types.exportDefaultDeclaration(addDisplayName(displayName, node.declaration)));
        }
      },
      CallExpression: function CallExpression(path) {
        var node = path.node;
        if (!isStyled(node)) return;
        var id; // crawl up the ancestry looking for possible candidates for displayName inference

        path.find(function (path) {
          if (path.isCallExpression()) {
            var _path$node$callee$pro;

            if (path.node.callee.name === 'object', ((_path$node$callee$pro = path.node.callee.property) === null || _path$node$callee$pro === void 0 ? void 0 : _path$node$callee$pro.name) === 'assign') {
              if (path.node.arguments.some(function (node) {
                return node.type === 'ObjectExpression' && node.properties.some(function (prop) {
                  return prop.key.name === 'displayName';
                });
              })) {
                // displayName already assigned
                return true;
              }
            }
          }

          if (path.isAssignmentExpression()) {
            id = path.node.left;
          } else if (path.isObjectProperty()) {
            id = path.node.key;
          } else if (path.isVariableDeclarator()) {
            id = path.node.id;
          } else if (path.isStatement()) {
            // we've hit a statement, we should stop crawling up
            return true;
          } // we've got an id! no need to continue


          if (id) return true;
        }); // ensure that we have an identifier we can inherit from

        if (!id) return; // foo.bar -> bar

        if (_core.types.isMemberExpression(id)) {
          id = id.property;
        } // identifiers are the only thing we can reliably get a name from


        if (_core.types.isIdentifier(id)) {
          path.replaceWith(addDisplayName(id.name, node));
        }
      }
    }
  };
});

exports["default"] = _default;